@@ -7,7 +7,7 @@ import numpy as np
 import astropy.io.fits as fits
 import astropy.units as u
 from astropy.coordinates import SkyCoord
-from astropy.table import Column, Table, hstack
+from astropy.table import Column, Table, hstack, vstack
 
 import starbug2
 from starbug2.utils import *
@@ -71,14 +71,6 @@ def sort_exposures(catalogues):
     return out
 
 def _match(cat1, cat2):
-    """
-    if not len(cat1) and len(cat2): 
-        n=len(cat2)
-        return np.array((range(n), sys.maxsize*np.ones(n)*u.degree, sys.maxsize*np.ones(n)))
-    if not len(cat2) and len(cat1): 
-        n=len(cat1)
-        return np.array((range(n), sys.maxsize*np.ones(n)*u.degree, sys.maxsize*np.ones(n)))
-    """
     _ra_cols= list( name for name in cat1.colnames if "RA" in name)
     _dec_cols= list( name for name in cat1.colnames if "DEC" in name)
     _ra= np.nanmean( tab2array( cat1, colnames=_ra_cols), axis=1) # this still breaks if the source isnt matched in all the columns, the 999 will increase the average
@@ -93,32 +85,47 @@ def _match(cat1, cat2):
 
     return skycoord2.match_to_catalog_3d(skycoord1)
 
-def generic_match(catalogues, threshold=0.25, add_src=True):
+def generic_match(catalogues, threshold=0.25, add_src=True, load=None, average=True):
     """
+    This matching works as a basic match. Everything is included and the column
+    names have _N appended to the end. 
+    INPUT:  catalogues=a list of Tables
+            threshold =separation threshold in units arcsecs
+            add_src= bool, append "new/unmatched" source to the end of the table
+            average= bool, try to calculate averages at the end
     """
     threshold=threshold*u.arcsec
     base=Table(None)
+    colnames=[]
+
+    if load==True:
+        n=sum( [len(c) for c in catalogues[1:]] )
+        load=loading(n,msg="matching",res=n/1000)
 
     for n,cat in enumerate(catalogues,1):
         if "Catalogue_Number" in cat.colnames: cat.remove_column("Catalogue_Number")
         if not len(base):
             tmp=cat.copy()
         else:
-            tmp=Table(tmp,dtype=[float]*len(tmp.colnames)).filled(np.nan) ## fill empty values with null
             idx,d2d,_=_match(base,cat)
             tmp=Table(np.full( (len(base),len(cat.colnames)), np.nan), names=cat.colnames)
 
             for src,IDX,sep in zip(cat,idx,d2d):
+                if load:
+                    load()
+                    load.show()
                 if (sep<=threshold) and (sep==min(d2d[idx==IDX])): ## GOOD MATCH
                     tmp[IDX]=src
                 elif add_src:   ##BAD MATCH / NEW SOURCE
                     tmp.add_row( src )
+
+        for name in tmp.colnames: 
+            if name not in colnames: colnames.append(name)
         tmp.rename_columns( tmp.colnames, ["%s_%d"%(name,n) for name in tmp.colnames] )
-        base=hstack((base,tmp))
-        #base=Table(base,dtype=[float]*len(base.colnames)).filled(np.nan) ## fill empty values with null
-    return base
-
 
+        base=fill_nan(hstack((base,tmp)))
+    if average: return finish_matching(base,colnames)
+    else: return base
 
 def dither_match(catalogues, threshold, colnames):
     """
@@ -167,6 +174,7 @@ def cascade_match(catalogues, threshold, colnames):
     ncol=len(colnames)
     base=Table( None, meta=catalogues[0].meta)#, names=colnames )
     load=loading(sum([len(cat) for cat in catalogues[1:]]),"matching")
+    fltr=find_filter(catalogues[0])
 
     for n,cat in enumerate(catalogues,1):
         if n==1:
@@ -194,9 +202,50 @@ def cascade_match(catalogues, threshold, colnames):
         tmp.rename_columns(colnames, list("%s_%d"%(name,n) for name in colnames))
         base=hcascade((base,tmp), colnames=colnames)
     base=Table(base,dtype=[float]*len(base.colnames)).filled(np.nan)
-    return finish_matching(base, colnames)
+    return finish_matching(base, colnames, fltr=fltr)
 
-def band_match(catalogues, threshold, colnames):
+def bootstrap_match(catalogues):
+    """
+    A more generalised band matching routine
+    A - B
+    B - C
+    C - D
+    """
+    print("THIS ISNT READY")
+
+    output=Table()
+
+    THRESHHOLD=0.3
+    filters= [c.meta.get("FILTER") for c in catalogues]
+    base=None
+
+    if len( catalogues ) >1:
+        base=catalogues[0]["RA","DEC","%s"%filters[0],"e%s"%filters[0]]
+
+        for n,cat in enumerate(catalogues[1:],1):
+            f=filters[n]
+            cat=cat["RA","DEC","%s"%f,"e%s"%f]
+            base=generic_match( (base,cat), add_src=True, average=False)
+
+            _f=filters[n-1]   #fix base colnames. Lower filter columns get finalised
+            base.rename_columns(("RA_1","DEC_1","%s_1"%_f,"e%s_1"%_f), 
+                                ("RA_%s"%_f,"DEC_%s"%_f,"%s"%_f,"e%s"%_f) ) 
+            for cn in base.colnames:
+                if cn[-2:]=="_1": base.rename_column(cn,cn[:-2])
+
+            #Get larger wavelength ready for next match
+            base.rename_columns( ("RA_2","DEC_2","%s_2"%f,"e%s_2"%f),
+                                 ("RA","DEC","%s"%f,"e%s"%f) )
+
+            print(n,len(catalogues))
+            if n==len(catalogues)-1: ##Final match
+                base.rename_columns(("RA","DEC"),("RA_%s"%f,"DEC_%s"%f) )
+
+    else:
+        perror("Must include more than one catalogue.\n")
+    return base
+
+def band_match(catalogues, colnames=("RA","DEC")):
     """
     Given a list of catalogues (with filter names in the meta data), match them
     in order of decreasing astrometric accuracy. 
@@ -204,8 +253,7 @@ def band_match(catalogues, threshold, colnames):
     taken as "correct". If a source is not resolved in this band, the next most 
     astrometrically accurate position is taken, i.e. F444W
     """
-    threshold=threshold*u.arcsec
-    colnames= list( name for name in catalogues[0].colnames if name in colnames)
+    #threshold=threshold*u.arcsec
     ### ORDER the tables into the correct order (increasing wavelength)
     tables=np.full( len(starbug2.filters), None)
     mask=np.full(len(starbug2.filters), False)
@@ -216,6 +264,10 @@ def band_match(catalogues, threshold, colnames):
                 tables[ii]=tab
                 mask[ii]=True
             else: perror("Unknown filter '%s' (skipping)..\n"%tab.meta["FILTER"])
+        elif (_tmp:=set(starbug2.filters.keys()) & set(tab.colnames)):
+            ii=list(starbug2.filters.keys()).index(_tmp.pop())
+            tables[ii]=tab
+            mask[ii]=True
         else: perror("Cannot find 'FILTER' in table meta (skipping)..\n")
     s="Bands: "
     for fltr,tab in zip(starbug2.filters.keys(),tables):
@@ -225,29 +277,47 @@ def band_match(catalogues, threshold, colnames):
 
     ### Match in increasing wavelength order
     base=Table(None)
-    load=loading(sum( [len(t) for t in tables[mask][1:]]),"matching", res=10)
+    load=loading(sum( [len(t) for t in tables[mask][1:]]),"matching", res=100)
     for fltr,tab in zip(starbug2.filters.keys(),tables):
         if not tab: continue
+        tab.remove_rows( np.isnan(tab[fltr]) ) ## removing empty magnitude rows
         load.msg="matching:%s"%fltr
-        if not len(base): tmp=tab[colnames].copy()
+        _colnames= list( name for name in tab.colnames if name in colnames)
+        if not len(base): 
+            tmp=tab[_colnames].copy()
         else:
             idx,d2d,_=_match(base,tab)
-            tmp=Table(np.full( (len(base),len(colnames)), np.nan), names=colnames)
+            tmp=Table(np.full( (len(base),len(_colnames)), np.nan), names=_colnames)
+            
+            ###################################
+            # Hard coding separations for now #
+            separation=0.06
+            f_id=list(starbug2.filters.keys()).index(fltr)
+            if f_id >= list(starbug2.filters.keys()).index("F277W"): separation=0.10
+            if f_id >= list(starbug2.filters.keys()).index("F560W"): separation=0.15
+            if f_id >= list(starbug2.filters.keys()).index("F1000W"): separation=0.20
+            if f_id >= list(starbug2.filters.keys()).index("F1500W"): separation=0.25
+
 
             for ii,(src,IDX,sep) in enumerate(zip(tab,idx,d2d)):
+                load.msg="matching:%s(%.2g\")"%(fltr,separation)
                 load();load.show()
-                if (sep<=threshold) and (sep==min(d2d[idx==IDX])):
-                    for name in colnames: tmp[IDX][name]=src[name]
+                if (sep<=separation*u.arcsec) and (sep==min(d2d[idx==IDX])):
+                    for name in _colnames: tmp[IDX][name]=src[name]
                 else:
-                    tmp.add_row(src[colnames])
+                    tmp.add_row(src[_colnames])
+            #tmp=generic_match((base,tab), threshold=threshold, add_src=True, load=load)
 
-        base=hstack(( base,tmp[["flux","eflux"]] ))
-        mag,magerr=flux2ABmag(tmp["flux"], tmp["eflux"],fltr)
-        base.add_column(mag,name=fltr)
-        base.add_column(magerr,name="e%s"%fltr)
+        #base=hstack(( base,tmp[["flux","eflux"]] ))
+        #mag,magerr=flux2ABmag(tmp["flux"], tmp["eflux"],fltr)
+        #base.add_column(mag,name=fltr)
+        #base.add_column(magerr,name="e%s"%fltr)
 
-        base.rename_column("flux","%s_flux"%fltr)
-        base.rename_column("eflux","%s_eflux"%fltr)
+        #base.rename_column("flux","%s_flux"%fltr)
+        #base.rename_column("eflux","%s_eflux"%fltr)
+
+        tmp.rename_column("flag","flag_%s"%fltr)
+        base=hstack(( base,tmp[[fltr,"e%s"%fltr,"flag_%s"%fltr]] ))#.filled(np.nan)
         base=Table(base,dtype=[float]*len(base.colnames)).filled(np.nan)
 
         ### Only keep the most astromectrically correct position
@@ -256,8 +326,15 @@ def band_match(catalogues, threshold, colnames):
             _mask=np.logical_and( np.isnan(base["RA"]), tmp["RA"]!=np.nan)
             base["RA"][_mask]=tmp["RA"][_mask]
             base["DEC"][_mask]=tmp["DEC"][_mask]
-    export_table(base,"/tmp/tab.fits")
-    return base
+
+    ## Sort out flags
+    flag=np.zeros(len(base),dtype=np.uint16)
+    for fcol in find_colnames(base,"flag"):
+        flag|=base[fcol].value.astype(np.uint16)
+        base.remove_column(fcol)
+    base.add_column(flag,name="flag")
+    
+    return base.filled(np.nan)
 
 def stage_match(stage2, stage3, threshold):
     """
@@ -287,12 +364,16 @@ def stage_match(stage2, stage3, threshold):
 
 
 
-def finish_matching(tab, colnames):
+def finish_matching(tab, colnames, fltr=None):
     """
     Averaging all the values. Combining source flags and building a NUM column
     """
     flags=np.full(len(tab),starbug2.SRC_GOOD, dtype=np.uint16)
     av=Table(np.full((len(tab),len(colnames)),np.nan), names=colnames)
+    if not fltr:
+        if not (fltr:=tab.meta.get("FILTER")):
+            if not (fltr:=find_filter(tab)):
+                fltr=None
 
     for name in colnames:
         all_cols=find_colnames(tab,name)
@@ -301,26 +382,37 @@ def finish_matching(tab, colnames):
         col=Column(None, name=name)
         ar=tab2array(tab, colnames=all_cols)
         if name=="flux":
-            col=Column(np.nanmean(ar,axis=1), name=name)
-            median=np.nanmedian(ar,axis=1)
+            col=Column(np.nanmedian(ar,axis=1), name=name)
+            mean=np.nanmean(ar,axis=1)
             if "stdflux" not in colnames: av.add_column(Column(np.nanstd(ar,axis=1),name="stdflux")) 
             ## if median and mean are >5% different, flag as SRC_VAR
-            flags[ np.abs(median-col)>(col/5.0)] |= starbug2.SRC_VAR
+            flags[ np.abs(mean-col)>(col/5.0)] |= starbug2.SRC_VAR
         elif name== "eflux":
             col=Column(np.sqrt(np.nansum(ar*ar, axis=1)), name=name)
         elif name=="stdflux": 
-            col=Column(np.nanmax(ar,axis=1),name=name)
+            col=Column(np.nanmedian(ar,axis=1),name=name)
         elif name=="flag":
             col=Column(flags, name=name)
             for fcol in ar.T: col|=fcol.astype(np.uint16)
+        elif name=="NUM":
+            col=Column(np.nansum(ar, axis=1), name=name)
         else: col=Column(np.nanmedian(ar, axis=1),name=name)
         
         av[name]=col
-    mag,magerr=flux2ABmag(av["flux"],av["eflux"], tab.meta["FILTER"])
-    av.add_column(mag,name=tab.meta["FILTER"])
-    av.add_column(magerr,name="e%s"%tab.meta["FILTER"])
-    narr= np.nansum( np.invert( np.isnan(tab2array(tab,find_colnames(tab,colnames[0])))),axis=1)
-    av.add_column(Column(narr, name="NUM"))
+    if len(set(["flux","eflux"])&set(av.colnames))==2:
+        _errcol= "stdflux" if "stdflux" in av.colnames else "e%s"%fltr
+        mag,magerr=flux2ABmag(av["flux"],av[_errcol], fltr)
+        if fltr is None: fltr="mag"
+
+        if fltr in av.colnames: av.remove_column(fltr)
+        if "e%s"%fltr in av.colnames: av.remove_column("e%s"%fltr)
+        av.add_column(mag,name=fltr)
+        av.add_column(magerr,name="e%s"%fltr)
+        perror("There was no zero point added here!\n")
+
+    if "NUM" not in av.colnames:
+        narr= np.nansum( np.invert( np.isnan(tab2array(tab,find_colnames(tab,colnames[0])))),axis=1)
+        av.add_column(Column(narr, name="NUM"))
     return (av,tab)
 
 
@@ -332,10 +424,3 @@ def remove_NUM(tab, N):
         pass#mask=tab["NUM"]
 
 
-if __name__=="__main__":
-    stage2=Table.read("/dat/1zw18/jwst/stage2/1zw18-apstage2.fits")
-    stage3=Table.read("/dat/1zw18/jwst/stage3/1Zw18-apstage3.fits")
-
-
-    tab=stage_match(stage2,stage3, 0.25*u.arcsec)
-    export_table(tab,fname="/tmp/tab.fits")
