@@ -1,6 +1,7 @@
 """
 Routines for Starbug
 """
+import os
 import sys
 import time
 import numpy as np
@@ -12,21 +13,20 @@ from skimage.feature import match_template
 
 from astropy.stats import sigma_clipped_stats, sigma_clip
 from astropy.coordinates import SkyCoord
-from astropy.table import Column, Table, QTable, hstack
+from astropy.table import Column, Table, QTable, hstack, vstack
 from astropy.modeling.fitting import LevMarLSQFitter
 from astropy.convolution import RickerWavelet2DKernel
 
 from photutils.background import Background2D, BackgroundBase
 from photutils.aperture import CircularAperture, CircularAnnulus, aperture_photometry
-from photutils.detection import StarFinderBase, DAOStarFinder
+from photutils.detection import StarFinderBase, DAOStarFinder, find_peaks
 from photutils.psf.groupstars import DAOGroup
-from photutils.psf import BasicPSFPhotometry
+from photutils.psf import PSFPhotometry, IntegratedGaussianPRF, SourceGrouper
 
 from photutils.datasets import make_model_sources_image, make_random_models_table
 from starbug2.utils import loading, printf, perror, warn
 from starbug2 import *
 
-
 class Detection_Routine(StarFinderBase):
     """
     Detection routine- called by starbug
@@ -37,26 +37,30 @@ class Detection_Routine(StarFinderBase):
     different set of sources
     """
     def __init__(self,  sig_src=5, sig_sky=3, fwhm=1,
-                        sharplo=0.2, sharphi=1, roundlo=-1, roundhi=1,
-                        verbose=0, cleansrc=1, bgd2d=1, boxsize=2):
+                        sharplo=0.2, sharphi=1, round1hi=1, round2hi=1,
+                        smoothlo=-np.inf, smoothhi=np.inf, ricker_r=1.0,
+                        verbose=0, cleansrc=1, dobgd2d=1, boxsize=2, doconvl=1):
         self.sig_src=sig_src
         self.sig_sky=sig_sky
         self.fwhm = fwhm
         self.sharphi=sharphi
         self.sharplo=sharplo
-        self.roundhi=roundhi
-        self.roundlo=roundlo
+        self.round1hi= round1hi if round1hi is not None else np.inf
+        self.round2hi= round2hi if round2hi is not None else np.inf
+        self.smoothlo= smoothlo if smoothlo is not None else -np.inf
+        self.smoothhi= smoothhi if smoothhi is not None else np.inf
+
+        self.ricker_r=ricker_r
         self.cleansrc=cleansrc
 
-        #self.match_threshold=u.Quantity(match_threshold)*u.dimensionless_unscaled
         self.catalogue=Table()
         self.verbose=verbose
 
-        self.bgd2d=bgd2d
+        self.dobgd2d=dobgd2d
         self.boxsize=boxsize
-        #self.filtersize=filtersize
+        self.doconvl=doconvl
 
-    def detect(self, data, bkg_estimator=None, xycoords=None):
+    def detect(self, data, bkg_estimator=None, xycoords=None, method=None):
         """
         The core detection step (DAOStarFinder
         INPUT:  
@@ -70,10 +74,16 @@ class Detection_Routine(StarFinderBase):
         if bkg_estimator:
             bkg=bkg_estimator(data)
 
-        _,_,std=sigma_clipped_stats(data,sigma=self.sig_sky)
-        daofind=DAOStarFinder(std*self.sig_src, self.fwhm, sharplo=self.sharplo, sharphi=self.sharphi,
-                roundlo=self.roundlo, roundhi=self.roundhi, peakmax=np.inf, xycoords=xycoords)
-        return daofind(data - bkg)
+        _,median,std=sigma_clipped_stats(data,sigma=self.sig_sky)
+        if method=="findpeaks":
+            return find_peaks(data-bkg, median+std*self.sig_src, box_size=11)
+
+        else:
+            roundhi=max((self.round1hi,self.round2hi))
+            find=DAOStarFinder(std*self.sig_src, self.fwhm, sharplo=self.sharplo, sharphi=self.sharphi,
+                roundlo=-roundhi, roundhi=roundhi, peakmax=np.inf, xycoords=xycoords)
+            return find(data - bkg)
+
 
     def _bkg2d(self, data):
         return Background2D(data, self.boxsize, filter_size=3).background
@@ -90,11 +100,9 @@ class Detection_Routine(StarFinderBase):
         base_sky=SkyCoord(x=base["xcentroid"], y=base["ycentroid"], z=np.zeros(len(base)), representation_type="cartesian")
         cat_sky=SkyCoord(x=cat["xcentroid"], y=cat["ycentroid"], z=np.zeros(len(cat)), representation_type="cartesian")
         _,separation,_=cat_sky.match_to_catalog_3d(base_sky)
-        for src,sep in zip(cat,separation.to_value()):
-            if sep>self.fwhm:#1 pixel?
-                base.add_row(src)
-                added+=1
-        return added
+        mask= separation.to_value() >self.fwhm
+        return vstack((base,cat[mask]))
+
 
     def find_stars(self, data, mask=None):
         """
@@ -119,18 +127,22 @@ class Detection_Routine(StarFinderBase):
         self.catalogue=self.detect(data)
         if self.verbose: printf("-> [PLAIN] pass: %d sources\n"%len(self.catalogue))
 
-        self.match(self.catalogue, self.detect(data, self._bkg2d))
-        if self.verbose: printf("-> [BGD2D] pass: %d sources\n"%len(self.catalogue))
+        if self.dobgd2d:
+            self.catalogue=self.match(self.catalogue, self.detect(data, self._bkg2d))
+            if self.verbose: printf("-> [BGD2D] pass: %d sources\n"%len(self.catalogue))
 
         ## 2nd order differential detection
-        kernel=RickerWavelet2DKernel(1)
-        conv=convolve(data, kernel)
-        corr=match_template(conv/np.amax(conv), kernel.array)
-        _detections=self.detect(corr)
-        _detections["xcentroid"]+=kernel.shape[0]//2
-        _detections["ycentroid"]+=kernel.shape[0]//2
-        self.match(self.catalogue, _detections)
-        if self.verbose: printf("-> [CONVL] pass: %d sources\n"%len(self.catalogue))
+        if self.doconvl:
+            kernel=RickerWavelet2DKernel(self.ricker_r)
+            conv=convolve(data, kernel)
+            corr=match_template(conv/np.amax(conv), kernel.array)
+            _detections=self.detect(corr, method="findpeaks")
+            if _detections:
+                _detections["x_peak"]+=kernel.shape[0]//2
+                _detections["y_peak"]+=kernel.shape[0]//2
+                _detections.rename_columns( ("x_peak","y_peak"),("xcentroid","ycentroid"))
+                self.catalogue=self.match(self.catalogue, _detections)
+            if self.verbose: printf("-> [CONVL] pass: %d sources\n"%len(self.catalogue))
 
         ## Now with xycoords DAOStarfinder will refit the sharp and round values at the detected locations
         #self.catalogue=self.detect(data, xycoords=np.array([self.catalogue["xcentroid"],self.catalogue["ycentroid"]]).T)#, clean=0)
@@ -138,17 +150,17 @@ class Detection_Routine(StarFinderBase):
         if tmp: self.catalogue=tmp
 
         mask=(~np.isnan(self.catalogue["xcentroid"]) & ~np.isnan(self.catalogue["ycentroid"]))
-        self.catalogue.remove_rows(~mask)
+        #self.catalogue.remove_rows(~mask)
 
-        mask = ((self.catalogue["sharpness"]>self.sharplo)
-               &(self.catalogue["sharpness"]<self.sharphi)
-               &(self.catalogue["roundness1"]>self.roundlo)
-               &(self.catalogue["roundness1"]<self.roundhi)
-               &(self.catalogue["roundness2"]>self.roundlo)
-               &(self.catalogue["roundness2"]<self.roundhi))
         if self.cleansrc: 
-            if self.verbose: printf("-> cleaning %d+ unlikley point sources\n"%sum(~mask))
-            self.catalogue.remove_rows(~mask)
+            mask &=((self.catalogue["sharpness"]>self.sharplo)
+                   &(self.catalogue["sharpness"]<self.sharphi)
+                   &(self.catalogue["roundness1"]> -self.round1hi)
+                   &(self.catalogue["roundness1"]<  self.round1hi)
+                   &(self.catalogue["roundness2"]> -self.round2hi)
+                   &(self.catalogue["roundness2"]<  self.round2hi))
+        if self.verbose: printf("-> cleaning %d unlikley point sources\n"%sum(~mask))
+        self.catalogue.remove_rows(~mask)
         
         if self.verbose: printf("Total: %d sources\n"%len(self.catalogue))
 
@@ -161,21 +173,26 @@ class APPhot_Routine():
     Aperture photometry called by starbug
     Given photometry radius, sky annuli radii rad_inner rad_outer
     """
-    def __init__(self, radius, sky_in, sky_out, encircled_energy=50, fit_radius=1, verbose=0):
+    def __init__(self, radius, sky_in, sky_out, verbose=0):
+        if sky_in < radius:
+            warn()
+            perror("Sky annulus radii must be larger than aperture radius.\n")
+            sky_in=radius+1
+
+        if sky_in >= sky_out: 
+            warn()
+            perror("Sky annulus outer radii must be larger than the inner.\n")
+            sky_out=sky_in+1
+
         self.radius=radius
         self.sky_in=sky_in
         self.sky_out=sky_out
-        self.encircled_energy=encircled_energy
-        self.fit_radius=fit_radius
         self.catalogue=Table(None)#, names=["ap_flux_r%d"%n for n in range(len(radii))]+["sky_median"])
-
         self.verbose=verbose
 
-
     def __call__(self, detections, image, **kwargs):
         return self.run(detections, image, **kwargs)
 
-    #def run(self, detections, image, apcorr=1.0, sig_sky=3):
     def run(self, detections, image, error=1, dqflags=None, apcorr=1.0, sig_sky=3):
         """
         Forced aperture photometry on a list of detections
@@ -199,22 +216,50 @@ class APPhot_Routine():
         mask=np.isnan(image)
 
         apertures=CircularAperture(pos,self.radius)
+        smooth_apertures=CircularAperture(pos, min(1.5*self.radius,self.sky_in))
         annulus_aperture=CircularAnnulus(pos, r_in=self.sky_in, r_out=self.sky_out)
-        phot=aperture_photometry(image, apertures, error=error, mask=mask)
         
-        self.catalogue=Table(np.full((len(pos),3),np.nan),names=("flux","eflux","sky"))
+        self.log("-> apertures: %.2g (%.2g - %.2g)\n"%(self.radius, self.sky_in, self.sky_out))
+        phot=aperture_photometry(image, (apertures,smooth_apertures), error=error, mask=mask)
+        self.catalogue=Table(np.full((len(pos),4),np.nan),names=("smoothness","flux","eflux","sky"))
 
-        self.log("-> calculating photometric errors\n")
-        for i,mask in enumerate(annulus_aperture.to_mask(method="center")):
-            dat=np.array(mask.multiply(image),dtype=float)
-            dat=sigma_clip(dat[dat>0 & np.isfinite(dat)], sigma=sig_sky)
-            if len(dat): ##sometimes all the surrounding pixels are nan OR above SIGSKY value??
-                #self.catalogue["sky"][i]=mode(dat)[0]
-                self.catalogue["sky"][i]=np.ma.median(dat)
+        self.log("-> calculating sky values\n")
+        masks=annulus_aperture.to_mask(method="center")
+        dat=list(map(lambda a:a.multiply(image),masks))
 
+        try: dat=np.array(dat).astype(float)
+        except:
+            ## Cases where the array is inhomegenoeus
+            ## If annulus reaches the edge of the image, it will create a mask the wrong shape
+            ## If for whatever reason the point lies outside the image, it will have None
+            ## in the list, this needs to be caught too
+            warn()
+            perror("Ran into issues with the sky annuli, trying to fix them..\n")
+            size=np.max( [np.shape(d) for d in dat if d is not None ])
+            for i,d in enumerate(dat):
+                if d is None: dat[i]=np.zeros((size,size))
+                elif (shape:=np.shape(d))!=(size,size):
+                    dat[i]=np.zeros((size,size))
+                    dat[i][:shape[0],:shape[1]]+=d
+            dat=np.array(dat)
 
-        self.catalogue["eflux"]=phot["aperture_sum_err"]
-        self.catalogue["flux"]=apcorr*(phot["aperture_sum"] - (self.catalogue["sky"]*apertures.area))
+        mask=(dat>0 & np.isfinite(dat))
+        dat[~mask]=np.nan
+        dat=sigma_clip(dat.reshape(dat.shape[0],-1), sigma=sig_sky,axis=1)
+        self.catalogue["sky"]=np.ma.median(dat,axis=1)
+        std=np.ma.std(dat,axis=1)
+
+        epoisson=phot["aperture_sum_err_0"]
+        esky_scatter= apertures.area*std**2
+        esky_mean=  (std**2 * apertures.area**2) / annulus_aperture.area
+
+        self.catalogue["eflux"]=np.sqrt( epoisson**2 +esky_scatter**2 +esky_mean**2)
+        self.catalogue["flux"]=apcorr*(phot["aperture_sum_0"] - (self.catalogue["sky"]*apertures.area))
+        
+        ######################
+        # Source "smoothness", the gradient of median pixel values within the two test apertures
+        ######################
+        self.catalogue["smoothness"] = (phot["aperture_sum_1"]/smooth_apertures.area) / (phot["aperture_sum_0"]/apertures.area)
 
         col=Column(np.full(len(apertures),SRC_GOOD), dtype=np.uint16, name="flag")
         if dqflags is not None:
@@ -236,18 +281,22 @@ class APPhot_Routine():
         Using CRDS apcorr table, fit a curve to the radius vs apcorr
         columns and then return aporr to respective input radius
         """
-        if not table_fname: return None
+        if not table_fname or not os.path.exists(table_fname): return 1
         tmp=Table.read(table_fname, format="fits")
-        t_apcorr=tmp[tmp["filter"]==filter][:-1] ## SKIPPING FINAL VALUE
+        
+        if "filter" in tmp.colnames:
+            t_apcorr=tmp[(tmp["filter"]==filter)]
+        else: t_apcorr=tmp
 
-        fn=lambda x,a,b,c: a*np.exp(-b*x)+c
-        popt,_=curve_fit(fn, t_apcorr["radius"], t_apcorr["apcorr"])
-        apcorr= fn( radius,*popt)
-        if verbose: printf("-> estimating aperture correction: %.3g..\n"%apcorr)
 
-        popt,_=curve_fit(fn,t_apcorr["radius"],t_apcorr["eefraction"])
-        eefrac= fn(radius,*popt)
-        if verbose: printf("-> effective encircled energy fraction: %.3g\n"%eefrac)
+        if "pupil" in t_apcorr.colnames:
+            t_apcorr=t_apcorr[ t_apcorr["pupil"]=="CLEAR"]
+        
+        apcorr= np.interp(radius, t_apcorr["radius"], t_apcorr["apcorr"])
+        if verbose: printf("-> estimating aperture correction: %.3g\n"%apcorr)
+
+        #eefrac= np.interp(radius, t_apcorr["radius"], t_apcorr["eefraction"])
+        #if verbose: printf("-> effective encircled energy fraction: %.3g\n"%eefrac)
         return apcorr
 
 
@@ -256,9 +305,12 @@ class APPhot_Routine():
         """
         Rather than fitting radius to the APCORR CRDS, use the closes Encircled energy value
         """
-        if not table_fname: return None
+        if not table_fname or not os.path.exists(table_fname): return 1
         tmp=Table.read(table_fname, format="fits")
-        t_apcorr=tmp[tmp["filter"]==filter]
+
+        if "filter" in tmp.colnames:
+            t_apcorr=tmp[(tmp["filter"]==filter)]
+        else: t_apcorr=tmp
 
         line=t_apcorr[(np.abs(t_apcorr["eefraction"]-encircled_energy)).argmin()]
         if verbose:
@@ -266,6 +318,22 @@ class APPhot_Routine():
             printf("-> using aperture correction: %f\n"%line["apcorr"])
 
         return line["apcorr"], line["radius"]
+    
+    def radius_from_encenrgy(filter, eefrac, table_fname):
+        """
+        """
+        if not table_fname or not os.path.exists(table_fname): return -1
+        t_apcorr=Table.read(table_fname, format="fits")
+
+        if len( set(["eefraction","radius"])&set(t_apcorr.colnames))!=2: return -1
+
+        if "filter" in t_apcorr.colnames: # Crop down table
+            t_apcorr=t_apcorr[(t_apcorr["filter"]==filter)]
+
+        if "pupil" in t_apcorr.colnames: # Crop down table
+            t_apcorr=t_apcorr[ t_apcorr["pupil"]=="CLEAR"]
+
+        return np.interp( eefrac, t_apcorr["eefraction"], t_apcorr["radius"])
 
     def log(self,msg):
         """
@@ -279,10 +347,12 @@ class APPhot_Routine():
 class BackGround_Estimate_Routine(BackgroundBase):
     """
     """
-    def __init__(self, sourcelist, boxsize=2, fwhm=2, verbose=0, bgd=None):#mask_r0=7, mask_r1=9
+    def __init__(self, sourcelist, boxsize=2, fwhm=2, sigsky=2, bgd_r=-1, verbose=0, bgd=None):#mask_r0=7, mask_r1=9
         self.sourcelist=sourcelist
         self.boxsize=boxsize
         self.fwhm=fwhm
+        self.sigsky=sigsky
+        self.bgd_r=bgd_r
         self.verbose=verbose
         self.bgd=bgd
         super().__init__()
@@ -298,16 +368,62 @@ class BackGround_Estimate_Routine(BackgroundBase):
         for i,mask in enumerate(apertures):
             peaks[i]=np.nanmax( mask.multiply(im) )
         return peaks
+    
+    def log(self,msg):
+        if self.verbose: printf(msg)
+
+
+    """
+    def calc_rlist(self,data):
+
+        _NCALC_BGDR=5.0
+        FUDGE=1/2.0
+        N=int(len(self.sourcelist)/_NCALC_BGDR)
+        ii=np.random.choice( len(self.sourcelist), size=N)
+        sources=self.sourcelist[ii]
+        shape=int(self.fwhm)*5
+        if not shape%2: shape+=1
+
+        mask=np.isnan(data)|np.isinf(data)
+        _,median,_=sigma_clipped_stats(data, sigma=3)
+        ro= 0.5*self.fwhm * np.sqrt( np.log(median/sources["flux"]))
+        ro*=FUDGE
+
+        with open("test.reg",'w') as fp:
+            for i in range(len(sources)):
+                if not np.isnan(ro[i]):
+                    src=sources[i]
+                    fp.write("fk5;circle %f %f %fi\n"%( src["RA"], src["DEC"], ro[i]))
+        x=np.array([min(sources["flux"]), max(sources["flux"])])
+    """
 
     def __call__(self, data, axis=None, masked=False):
         if self.sourcelist is None or data is None: return self.bgd
         _data=np.copy(data)
         X,Y=np.ogrid[:data.shape[1], :data.shape[0]]
-        peaks=self.calc_peaks(data)
 
-        rlist=np.sqrt(peaks**0.7)*self.fwhm/1.5 ## <-- that works but hmm
+        FUDGE=1
+        DEFAULT_R=2*self.fwhm
+
+        if self.bgd_r and self.bgd_r>0:
+            self.log("-> using BGD_R=%g masking aperture radii\n"%self.bgd_r)
+            rlist=self.bgd_r*np.ones(len(self.sourcelist)) 
+
+        else:
+            #peaks=self.calc_peaks(data)
+            #rlist=np.sqrt(peaks**0.7)*self.fwhm/1.5 ## <-- that works but hmm
+            if "flux" in self.sourcelist.colnames:
+                self.log("-> calculating source aperture mask radii\n")
+                _,median,_=sigma_clipped_stats(data,sigma=self.sigsky)
+                rlist= FUDGE * self.fwhm/2 * np.sqrt( np.log( median/self.sourcelist["flux"]) )
+                rlist[np.isnan(rlist)]=DEFAULT_R
+            else:
+                warn()
+                perror("Unable to caluclate aperture mask sizes, add '-A' to starbug command.\n")
+                rlist=DEFAULT_R*np.ones(len(self.sourcelist))
+
         D=50
-        load=loading(len(self.sourcelist), msg="masking sources", res=10)
+        load=loading(len(self.sourcelist), msg="masking sources", res=10)#len(self.sourcelist)/1000)
         for r,src in zip(rlist,self.sourcelist):
 
             rin=1.5*r
@@ -342,13 +458,42 @@ class _grouping(DAOGroup):
     Overwritten DAOGroup that just holds the number of groups
     for use in verbose loading of psfphot routine
     >>> This is now a bit redundant after photoutils added progress_bar=true
+
+    Issue:6 recursion during fitting.
+        >>> the fitting seems to include this recursive step within a given source of sources
+            if the group contains more members than the system recursion limit then it will
+            unceremonially crash on a recursion error.
+        >>> for now at least, I will give a warning that this would have occurred, but then 
+            ill override the recursion limit and avoid the crash. Hopefully
     """
+    logfile=None
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.ngroups=0
+        if os.getenv("SBIIDEBUG"):
+            self.logfile=open(os.getenv("SBIIDEBUG"),'a')
+            self.logfile.write("# PSF Source Grouping\n")
+
     def __call__(self, *args):
         res=super().__call__(*args)
         self.ngroups=max(res["group_id"])
+
+        #### hacking recursion error
+        for gid in set(res["group_id"]):
+            n_members=sum(res["group_id"]==gid)
+            if self.logfile:
+                self.logfile.write("GID:%d (%d)\n"%(gid,n_members))
+
+            ## It seems to not quite hit the recursion limit
+            ## Crashed on 980 with a limit of 1000, so im going to try 90%
+            if n_members > (0.9*sys.getrecursionlimit()):
+                warn()
+                perror("This run will exceed the recursion depth of the system. "
+                       "Starbug will intervene and override the recursion limit but "
+                       "the parameter \"CRIT_SEP\" should be reduced to avoid this.\n"
+                       "Setting recursion limit %d -> %d\n"%(sys.getrecursionlimit(), int(2.0*n_members)))
+                sys.setrecursionlimit(int(2.0*n_members))
+        if self.logfile: self.logfile.close()
         return res
 
 class _fitmodel(LevMarLSQFitter):
@@ -368,126 +513,61 @@ class _fitmodel(LevMarLSQFitter):
         return super().__call__(*args,**kwargs)
 
 
-class PSFPhot_Routine(BasicPSFPhotometry):
+class PSFPhot_Routine(PSFPhotometry):
     """
     PSF Photometry routine called by starbug
     """
-    def __init__(self, crit_separation, psf_model, fitshape,
-            force_fit=False, dposition_threshold=2, background=None, verbose=1):
+    def __init__(self, psf_model, fitshape, apphot_r=3,
+            force_fit=False, background=None, verbose=1):
 
         self.verbose=verbose
         self.force_fit=force_fit        
-        self.dpos_thresh=dposition_threshold
+        self.background=background
 
-        group_maker=_grouping(crit_separation=crit_separation)
-        bkg_estimator=BackGround_Estimate_Routine(None, bgd=background)
-        fitter=_fitmodel(grouper=group_maker, verbose=verbose)
+        #group_maker=_grouping(crit_separation=8)#crit_separation)
+        #bkg_estimator=BackGround_Estimate_Routine(None, bgd=background)
+        #fitter=_fitmodel(grouper=group_maker, verbose=verbose)
         
         if force_fit:
-            psf_model.fixed.update( {"x_0":True,"y_0":True} )
-            if fitter.load is not None: fitter.load.msg+=" (forced)"
+            psf_model.x_0.fixed=True
+            psf_model.y_0.fixed=True
+            #psf_model.fixed.update( {"x_0":True,"y_0":True} )
+            #if fitter.load is not None: fitter.load.msg+=" (forced)"
 
-        super().__init__(group_maker=group_maker, bkg_estimator=bkg_estimator,
-                psf_model=psf_model, fitshape=fitshape,
+        super().__init__(psf_model=psf_model, fit_shape=fitshape, finder=None,
+                progress_bar=verbose, aperture_radius=apphot_r,
+                grouper=SourceGrouper(8))
+                #localbkg_estimator=bkg_estimator, fitter=fitter)
+        """
+        super().__init__(grouper=group_maker, localbkg_estimator=bkg_estimator,
+                psf_model=psf_model, fit_shape=fitshape,
                 finder=None, fitter=fitter)
+        """
 
-    #def _bkg(self, axis=None,masked=None):
-        #return self.background
-
-    def do_photometry(self, image, mask=None, init_guesses=None, progress_bar=False):
+    def __call__(self,*args,**kwargs): return self.do_photometry(*args,**kwargs)
+    def do_photometry(self, image, mask=None, init_params=None, progress_bar=False):
         """
         """ 
-        _cat=None
-        _fixcat=None
 
-        if init_guesses is None:
+        if init_params is None or len(init_params)==0:
             perror("Must include source list\n")
             return None
 
-        if self.verbose: printf("-> fitting %d sources\n"%len(init_guesses))
-        cat=super().do_photometry(image, mask=mask, init_guesses=init_guesses, progress_bar=False)
+        if self.background is not None: image=image-self.background
+        if self.verbose: printf("-> fitting %d sources\n"%len(init_params))
+        cat=super().__call__(image, mask=mask, init_params=init_params)
 
-        if "flux_unc" not in cat.colnames:
+        d=np.sqrt((cat["x_init"]-cat["x_fit"])**2.0 + (cat["y_init"]-cat["y_fit"])**2.0)
+        cat.add_column(Column(d,name="xydev"))
+
+        if "flux_err" not in cat.colnames:
             cat.add_column(Column(np.full(len(cat),np.nan), name="eflux"))
             perror("NO ERRORS??\n")
-        else: cat.rename_column("flux_unc","eflux")
-
-        return cat
-
-class Cleaning_Routine(object):
-    """
-    docstring...
-    """
-    sharp_mu=0
-    sharp_sig=0
-
-    round1_mu=0
-    round1_sig=0
-
-    round2_mu=0
-    round2_sig=0
-
-    verbose=1
-
-    def __init__(self, cat, verbose=1 ):
-        self.cat=cat
-        self.verbose=verbose
-        self.fit()
-
-    def log(self,msg):
-        if self.verbose: printf(msg)
-
-    def fit(self):
-        if "sharpness" in self.cat.colnames:
-            self.sharp_mu, self.sharp_sig   = norm.fit(self.cat["sharpness"])
-            self.log("fit 'sharpness': mu=%.2g sig=%.2g\n"%(self.sharp_mu, self.sharp_sig))
-
-        if "roundness1" in self.cat.colnames:
-            self.round1_mu, self.round1_sig = norm.fit(self.cat["roundness1"])
-            self.log("fit 'roundness1': mu=%.2g sig=%.2g\n"%(self.round1_mu, self.round1_sig))
-
-        if "roundness2" in self.cat.colnames:
-            self.round2_mu, self.round2_sig = norm.fit(self.cat["roundness2"])
-            self.log("fit 'roundness2': mu=%.2g sig=%.2g\n"%(self.round2_mu, self.round2_sig))
-
-    def _remove_rows(self,cat,indices):
-        length=len(cat)
-        cat.remove_rows(indices)
-        if "id" in cat.colnames: cat.replace_column("id", Column(range(1,1+len(cat))))
-        return length-len(cat)
-
-    def run(self, mag_unc=1, sharp_sig_hi=2, sharp_sig_lo=2,
-                                        round_sig_hi=2, round_sig_lo=2):
-        """
-        if "ap_flux" in self.cat.colnames:
-            dr=self._remove_rows(self.cat, self.cat["ap_flux"]<0)
-            self.log("-> cut %d sources with ap_phot flux<0\n"%dr)
-        """
-
-        if "mag_unc" in self.cat.colnames:
-            dr=self._remove_rows(self.cat, self.cat["mag_unc"]>mag_unc)
-            self.log("-> cut %d sources with mag_error>%g\n"%(dr,mag_unc))
-
-        if "sharpness" in self.cat.colnames:
-            dr =self._remove_rows(self.cat, self.cat["sharpness"]>(self.sharp_mu + (sharp_sig_hi*self.sharp_sig)))
-            dr+=self._remove_rows(self.cat, self.cat["sharpness"]<(self.sharp_mu - (sharp_sig_lo*self.sharp_sig)))
-            self.log("-> cut %d sources on sharpness\n"%dr)
-
-        if "roundness1" in self.cat.colnames:
-            dr =self._remove_rows(self.cat, self.cat["roundness1"]>(self.round1_mu + (round_sig_hi*self.round1_sig)))
-            dr+=self._remove_rows(self.cat, self.cat["roundness1"]<(self.round1_mu - (round_sig_lo*self.round1_sig)))
-            self.log("-> cut %d sources on roundness1\n"%dr)
-
-        if "roundness2" in self.cat.colnames:
-            dr =self._remove_rows(self.cat, self.cat["roundness2"]>(self.round2_mu + (round_sig_hi*self.round2_sig)))
-            dr+=self._remove_rows(self.cat, self.cat["roundness2"]<(self.round2_mu - (round_sig_lo*self.round2_sig)))
-            self.log("-> cut %d sources on roundness2\n"%dr)
-
-        self.log("final catalogue size: %d\n"%len(self.cat))
-        return self.cat
-
-
-
+        else: cat.rename_column("flux_err","eflux")
+        
+        keep=["x_fit","y_fit","flux_fit","eflux","xydev","qfit"]
+        #cat=hstack(( init_params, cat[keep]))
+        return hstack((init_params, cat[keep]))
 
 class ArtificialStar_Routine(object):
     """
@@ -650,7 +730,7 @@ class SourceProperties:
         daofind=DAOStarFinder(-np.inf, fwhm, sharplo=-np.inf, sharphi=np.inf, roundlo=-np.inf, roundhi=np.inf, xycoords=xycoords, peakmax=np.inf)
         return daofind._get_raw_catalog(self.image).to_table()
 
-        
+
 
 
 
