@@ -1,7 +1,7 @@
 import time
 import os, sys, numpy as np
 from parse import parse
-from astropy.table import Table,hstack,Column
+from astropy.table import Table,hstack,Column,MaskedColumn
 from astropy.io import fits
 import starbug2
 import requests
@@ -34,7 +34,7 @@ class loading(object):
         self.setlen(N)
         self.msg=msg
         self.startime=time.time()
-        self.res=res
+        self.res=int(res)
 
     def setlen(self,n): 
         self.N=abs(n)
@@ -134,8 +134,8 @@ def load_params(fname):
                 if line[0] in "# \t\n":
                     continue
                 key,value,_=parse("{}={}//{}\n",line)
-                key=key.strip()
-                value=value.strip()
+                key=key.strip().rstrip()
+                value=value.strip().rstrip()
                 try:
                     if '.' in value: value=float(value)
                     else: value=int(value)
@@ -143,13 +143,33 @@ def load_params(fname):
                     pass
 
                 ## Special case values
-                if key in ("AP_FILE","BGD_FILE","PSF_FILE"): value=os.path.expandvars(value)
+                if key in ("OUTPUT", "AP_FILE","BGD_FILE","PSF_FILE"): value=os.path.expandvars(value)
                 config[key]=value
     else:
         perror("config file \"%s\" does not exist\n"%fname)
         config=None
     return config
 
+def parse_unit(raw):
+    """
+    Take a value with the ability to be cast into several units and parse it
+    i.e. 123p -> 123 'pixels'
+    """
+    recognised={'p':starbug2.PIX, 's':starbug2.ARCSEC, 'm':starbug2.ARCMIN, 'd':starbug2.DEG}
+    value=None
+    unit=None
+    if raw:
+        try: 
+            value=float(raw)
+            unit=None
+        except:
+            try:
+                value=float(raw[:-1])
+                unit=recognised.get(raw[-1])
+            except:
+                perror("unable to parse '%s'\n"%raw)
+    return value,unit
+
 def tab2array(tab,colnames=None):
     """
     Returns the contents of the table as a notmal 2D numpy array
@@ -165,7 +185,45 @@ def tab2array(tab,colnames=None):
 def export_table(table, fname=None, header=None):
     if table:
         if not fname: fname="/tmp/starbug.fits"
-        fits.BinTableHDU(data=reindex(table), header=header).writeto(fname, overwrite=True)
+        btab=fits.BinTableHDU(data=reindex(table), header=header).writeto(fname, overwrite=True, output_verify="fix")
+
+def import_table(fname):
+    """
+    """
+    tab=None
+    if os.path.exists(fname):
+        if os.path.splitext(fname)[1]==".fits":
+            tab=Table.read(fname,format="fits")
+            tmp=Table()
+
+            names=[]
+            for index in range(len(tab.dtype.names)):
+                if tab.dtype[index].kind!='f':
+                    name=tab.colnames[index]
+                    names.append(name)
+                    tmp.add_column( tab[name] )
+            tab.remove_columns( names )
+            tab=tab.filled(np.nan)
+            if tmp: tab=hstack((tab,tmp))
+
+        else: perror("Table must fits format\n")
+    else: perror("Unable to locate \"%s\"\n"%fname)
+    return tab
+
+def fill_nan(table):
+    """
+    Fill empty values in table with nans
+    This is useful for tables that have columns that
+    dont support nans (e.g. starbug flag). These will be set to zero instead
+    """
+    for i,name in enumerate(table.colnames):
+        #fill_val=np.nan
+        #if table.dtype[i].kind!='f': fill_val=0
+        fill_val=np.nan if table.dtype[i].kind=='f' else 0
+        if type(table[name])==MaskedColumn: table[name]=table[name].filled(fill_val)
+        #print(table.dtype[i], name,fill_val)
+
+    return table
 
 def find_colnames(tab, basename):
     """
@@ -285,15 +343,41 @@ def reindex(table):
     table.add_column(column,index=0)
     return table
 
+def colour_index(table,keys):
+    """
+    Allow table indexing with A-B
+    """
+    out=Table()
+    for key in keys:
+        if key in table.colnames: out.add_column(table[key])
+        elif '-' in key:
+            a,b=key.split('-')
+            out.add_column(table[a]-table[b],name=key)
+    return out
+
 def get_MJysr2Jy_scalefactor(ext):
     """
     Find the unit scale factor to convert an image from MJy/sr to Jy
     """
     scalefactor=1
-    if ext.header["BUNIT"]=="MJy/sr":
-        scalefactor=1e6*float(ext.header["PIXAR_SR"])
+    if ext.header.get("BUNIT")=="MJy/sr":
+        if "PIXAR_SR" in ext.header:
+            scalefactor=1e6*float(ext.header["PIXAR_SR"])
     return scalefactor
 
+def find_filter(table):
+    lst=list(set(table.colnames)&set(starbug2.filters.keys()))
+    if lst: return lst.pop()
+    else: return None
+
+
 
 if __name__ == "__main__":
-    print(starbug2.logo)
+    print(parse_unit(""))
+    print(parse_unit("10p"))
+    print(parse_unit("10 p"))
+    print(parse_unit("10 D"))
+    print(parse_unit("10"))
+    print(parse_unit("p10"))
+
+
